import 'package:http/http.dart';
import 'package:suggestion_providers/src/base/provider_base.dart';
import 'package:suggestion_providers/src/providers/ask.dart';
import 'package:suggestion_providers/src/providers/brave.dart';
import 'package:suggestion_providers/src/providers/duckduckgo.dart';
import 'package:suggestion_providers/src/providers/ecosia.dart';
import 'package:suggestion_providers/src/providers/google.dart';
import 'package:suggestion_providers/src/providers/search_encrypt.dart';
import 'package:suggestion_providers/src/providers/searx.dart';
import 'package:suggestion_providers/src/providers/startpage.dart';

import 'providers/wolfram_alpha.dart';

/// Get search suggestions (typeahead) from search engines, like google, duckduckgo and more.
class Suggestions {
  final Client _client;

  /// All providers.
  late var all = <Provider Function()>[
    () => google,
    () => duckduckgo,
    () => ecosia,
    () => brave,
    () => startPage,
    () => ask,
    () => wolframAlpha,
    () => searchEncrypt,
  ];

  /// Get provider by position from [all].
  /// All it does is that it `call` it.
  Provider provider(int pos) => all[pos].call();

  /// All searx instances that i tested.
  /// There alot more, you can pass custom one in the [searx] method.
  static var searxInstances = [
    "https://searx.be",
    "https://www.webrats.xyz",
    "https://searx.hummel-web.at",
    "https://searx.tiekoetter.com",
  ];

  Suggestions([Client? client]) : _client = client ?? Client();

  late final google = Google(_client);

  late final duckduckgo = Duckduckgo(_client);

  late final ecosia = Ecosia(_client);

  late final brave = Brave(_client);

  late final startPage = StartPage(_client);

  late final ask = Ask(_client);

  late final wolframAlpha = WolframAlpha(_client);

  late final searchEncrypt = SearchEncrypt(_client);

  /// Searx instance.
  /// [base] for the base instance (https://searx.me), default is the first instance found in [searxInstances].
  /// [name] name of the provider, default is `Searx`.
  Searx searx({String? base, String name = "Searx"}) =>
      Searx(base ?? searxInstances.first, _client, name);
}
